;; -*- mode: common-lisp -*-
;;
;; Copyright (C) 2006 Franz Inc, Oakland, CA.  All rights reserved.
;;
;; This code is free software; you can redistribute it and/or
;; modify it under the terms of the version 2.1 of
;; the GNU Lesser General Public License as published by 
;; the Free Software Foundation, as clarified by the Franz
;; preamble to the LGPL found in
;; http://opensource.franz.com/preamble.html.
;;
;; This code is distributed in the hope that it will be useful,
;; but without any warranty; without even the implied warranty of
;; merchantability or fitness for a particular purpose.  See the GNU
;; Lesser General Public License for more details.
;;
;; Version 2.1 of the GNU Lesser General Public License can be
;; found at http://opensource.franz.com/license.html.
;; If it is not present, you can access it from
;; http://www.gnu.org/copyleft/lesser.txt (until superseded by a newer
;; version) or write to the Free Software Foundation, Inc., 59 Temple
;; Place, Suite 330, Boston, MA  02111-1307  USA
;;
;; $Id$

(in-package :user)

(defvar *weather-checker-running* nil)

(defparameter *weather-sites* '("koak" "ksfo"))

(defparameter *weather-updated* nil) 
(defparameter *weather-updating* nil)
(defparameter *weather-name* (make-hash-table :test #'equalp))
(defparameter *weather-temp* (make-hash-table :test #'equalp))
(defparameter *weather-sky*  (make-hash-table :test #'equalp))

(defun weather-monitor ()
  ;; start weather monitor if it isn't running already
  (if* (not *weather-checker-running*)
     then (setq *weather-checker-running* t)
	  (mp:process-run-function "Weather Monitor"
	    #'weather-checker '*weather-sites*)
     else ;; do a process-reset on it in case *weather-sites* updated
	  (ignore-errors
	   (mp:process-reset (mp:process-name-to-process "Weather Monitor")))
	  ))

(defun weather-checker (sites-var)
  ;; sites-var is a variable containing a list of
  ;; (name-of-site1 name-of-site2 ... name-of-siteN)
  ;;
  (flet ((get-wx (site)
	   (let (temp sky name)
	     (multiple-value-bind (ans error)
		 (ignore-errors (get-weather site))
	       (if* (or error (null ans))
		  then (format *initial-terminal-io*
			       "got error ~a when checking weather" error)
		       nil
		  else ;  got an answer
		       (setq ans (delete #\return ans)) 
		       (multiple-value-bind (ok whole val)
			   (match-re "Sky conditions: (.*)"  ans)
			 (declare (ignore whole))
			 (if* ok then (setq sky val)))
	      
		       (multiple-value-bind (ok whole val)
			   (match-re "Temperature: (.* F)"  ans)
			 (declare (ignore whole))
			 (if* ok then (setq temp val)))
		       
		       (multiple-value-bind (ok whole val)
			   (match-re 
			    (format nil "\(.*\) \\(~a\\)"
				    (string-upcase site)) ans)
			 (declare (ignore whole))
			 (if* ok 
			    then (let ((split-val (split-re ",\\W+" val)))
				   (if* (> (length split-val) 2)
				      then (setq name
					     (concatenate 'string
					       (first split-val) ", "
					       (third split-val)))
					   else (setq name val)))))
		       (values name temp sky))))))
    
    (loop
      (unwind-protect
	  (progn (setq *weather-updating* t)
		 (dolist (site (symbol-value sites-var))
		   (multiple-value-bind (name temp sky) (get-wx site)
		     (when name (setf (gethash site *weather-name*) name))
		     (when temp (setf (gethash site *weather-temp*) temp))
		     (when sky (setf (gethash site *weather-sky*) sky))
		     (when (or temp sky)
		       (setf *weather-updated* (get-universal-time))))))
      
	(setq *weather-updating* nil))
      ;; we want to check at 5 minutes after the hour
      (multiple-value-bind (secs min)
	  (decode-universal-time (get-universal-time))
	(declare (ignore secs))
	(let ((diff (- 67 min)))
	  (sleep (* diff 60)))))))

(defun get-weather (site)
  (let ((str (net.ftp.client:open-ftp-stream 
	      (format nil "data/observations/metar/decoded/~a.TXT"
		      (string-upcase (string site)))
	      :host "tgftp.nws.noaa.gov"
	      :user "anonymous"
	      :password "sample@user"
	      :mode :text
	      :passive t)))
    (unwind-protect 
	(let ((buffer (make-string 5000)))
	  (let ((i 0))
	    (loop
	      (let ((end (read-sequence buffer str :start i)))
		(if* (<= end i)
		   then ;; end of file
			(return)
		   else (setq i end))))
	    (subseq buffer 0 i)))
      ;; cleanup
      (close str))))

(def-clp-function weather_display-weather (req ent args body)
  (declare (ignore req ent args body))
  (dolist (site *weather-sites*)
    (let* ((sky (gethash site *weather-sky* "unknown"))
	   (temp (gethash site *weather-temp* "unknown"))
	   (name (gethash site *weather-name* "unknown"))
	   (temp-num (ignore-errors (parse-integer temp :junk-allowed t)))
	   (temp-class (cond ((and temp-num (>= temp-num 80)) "hot")
			     ((and temp-num (<= temp-num 60)) "cold")
			     (t "warm"))))
      (html ((:div class "weather-entry") 
	     "The weather in " 
	     ((:span class "location") (:princ-safe name))
	     " is "
	     (:princ-safe sky)
	     " with a temp of "
	     ((:span class temp-class) (:princ-safe temp)) :br)))))

(def-clp-function weather_lastupdate (req ent args body)
  (declare (ignore req ent args body))
  (html
   (:princ-safe (excl.osi:ctime *weather-updated*))))

(def-clp-function weather_isupdating (req ent args body)
  (declare (ignore req ent args body))
  (when *weather-updating*
    (html
     ((:div class "status")
      "Weather reports are updating.  "
      "You may notice strange results until the process completes."))))

(push 'weather-monitor *reload-hook*)
